/*

This code may help developers define the datum triggers that TurnHandlers call.
For use of a TurnHandler's own procs, refer to the basic and advanced dm files.

When run, this simple text demo allows the tester to see what the code does step
by step.  Can this library be used with graphics?  Absolutely.  One has little
to do with the other.

*/



//	We need a TurnHandler.
var/global/datum/TurnHandler/gTurnHandler	= new



//	We need something to take a turn, but it doesn't have to be a mob.  I'm just
//	using one for this demo because it's what many are used to.
mob/TurnsDemo

	var
		//	There might be more advanced ways of keeping track of how many turns
		//	a player has.  However, this will do for the demo.  I'll explain as
		//	this goes on.
		turnsTaken			= 0//	The amount of turns taken this round.
		turnsThisRound		= 1//	The total number of turns to take

		maxTurnsPerRound	= 1//	These are just values to help show that
		minTurnsPerRound	= 1//	the number of turns per round can vary.

	//--------------------------------------------------------------------------
	//	To take a turn, a datum must first have an amount of turns in the round.
	//--------------------------------------------------------------------------

	turns_TurnsLeftInRound(datum/TurnHandler/Handler,Round,Turn)
		//	This returns how many turns the datum has left at round Round and
		//	turn Turn.  Your method may vary, but I find this to be easiest.
		return (src.turnsThisRound - src.turnsTaken)

	turns_RoundStarted(datum/TurnHandler/Handler,Round,Turn)
		//	Handler is beginning round Round on turn Turn.  We can use this
		//	opportunity to reinitialize the number of turns to take.
		src.turnsTaken	= 0
		//	This library allows a variable amount of turns.  You'd probably make
		//	it depend on some reasonable stat, but a random value should be good
		//	enough for this demo.  (Testers can input demo values later.)
		src.turnsThisRound	= rand(src.minTurnsPerRound,src.maxTurnsPerRound)

		// Let's show the testers what just happened.
		world << "[src]:\t(Round [Round], Turn [Turn])\t[Round]\th ROUND STARTED."

	turns_RoundEnded(datum/TurnHandler/Handler,Round,Turn)
		//	Handler finished round Round on turn Turn.  We probably could have
		//	reset our turns here if desired.  Let's just show the testers that
		//	the trigger was called.
		world << "[src]:\t(Round [Round], Turn [Turn])\t[Round]\th ROUND ENDED."



	//--------------------------------------------------------------------------
	//	We'll then want to keep track when we spend turns.
	//--------------------------------------------------------------------------

	// Handler has told the datum to begin turn Turn of round Round.
	turns_BeginTurn(datum/TurnHandler/Handler,Round,Turn)
		//	We want the number of turns this round to decrease when a turn is
		//	taken.  Above, we subtracted turnsTaken from turnThisRound in order
		//	to find the number of rounds left.
		src.turnsTaken++

		//	In a real game, the player would probably be given access to some
		//	commands at this point or just be forced to take an action.  That's
		//	specific to the game in question so we'll just display the trigger
		//	call in this demo.
		world << "[src]:\t(Round [Round], Turn [Turn])\tStarting [src.turnsTaken]\th of [src.turnsThisRound] turn\s..."


		//	In a real game, a player might take a specific action to end the
		//	turn or maybe the turn would be ended after a certain amount of
		//	time.  Here, we're ending the turn immediately because all we wanted
		//	was to view the test output.  (BTW, if you're looking for timed
		//	turns, you may wish to try integrating one of the many timer
		//	libraries available  on BYOND.)
		Handler.StopWaitingForTurn(src,Round,Turn,FALSE)



	//--------------------------------------------------------------------------
	//	In a game where only one player can take a turn at a time, the following
	//	proc could be a decent place to remove this player's access to commands.
	//	However, there are none to remove in this demo.
	//--------------------------------------------------------------------------

	// Handler will no longer wait for the datum to finish turn Turn of round Round.
	turns_DoneWaitingForLastTurn(datum/TurnHandler/Handler,Round,Turn)
		world << "[src]:\t(Round [Round], Turn [Turn])\t...No more waiting for last turn."



	//--------------------------------------------------------------------------
	//	It is rarely necessary to define the following procs.  If we were
	//	working with multiple TurnHandlers at the same time then we might care,
	//	but we're not in this demo so we'll simply display the trigger calls.
	//--------------------------------------------------------------------------

	// The datum has been added to handler Handler at turn Turn of round Round.
	turns_AddedToHandler(datum/TurnHandler/Handler,Round,Turn)
		world << "[src]:\t(Round [Round], Turn [Turn])\tAdded to handler."

	// The datum is no longer involved with Handler as of turn Turn of round Round.
	// Note that the datum will not be told to end any ongoing turns.
	turns_RemovedFromHandler(datum/TurnHandler/Handler,Round,Turn)
		world << "[src]:\t(Round [Round], Turn [Turn])\tRemoved from handler."






//==============================================================================
//	A very small amount might be learned from the calls to TurnHandler's
//	AddPlayer(), RemovePlayer(), and StartTurn() procs.  However, the verbs
//	below are intended as merely a quick interface to test the demo.
//
//	I'd read the basic/advanced help files instead of paying much attention to
//	anything below this line.
//==============================================================================



client

	New()
		..()
		src	<< {"
This simple text demo allows the tester to see what ACWraith.Turns does step
by step.  Can this library be used with graphics?  Absolutely.  One has little
to do with the other.
"}

	verb
		//----------------------------------------------------------------------
		// Allow the tester to add a player to the turn handler and name it.
		add_player()
			set category	= "TEST"

			var
				mob/TurnsDemo/player	= new
				N	= input(usr,"What will you call the player?","Add Player","Nobody") as text

			player.name	= N

			var
				min	= 0
				max	= 0

			while (min < 1)
				min	= input(usr,"At LEAST how many turns will [N] have per round?","Add Player",1) as num
			player.minTurnsPerRound	= round(min)

			while (max < 1)
				max	= input(usr,"At MOST how many turns will [N] have per round?","Add Player",1) as num
			player.maxTurnsPerRound	= round(max)

			gTurnHandler.AddPlayer(player)

		//----------------------------------------------------------------------
		// Allow the tester to remove a specific player from the turn handler.
		remove_player()
			set category	= "TEST"

			var/player	= \
				input(usr,"Remove which player?","Remove Player",null) in gTurnHandler.GetPlayers()

			gTurnHandler.RemovePlayer(player)

			// We don't need the player anymore so get rid of it.
			del player

		//----------------------------------------------------------------------
		// Allow the tester to process a number of turns.
		take_turns()
			set category	= "TEST"

			var/turns	= input(usr,"How many turns at most?","Take Turns",1) as num

			world << "********************"

			while(turns > 0)
				gTurnHandler.StartTurn()
				turns--

			world << "********************"
